package com.fastbee.deviceData.api.renke.service;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.fastbee.deviceData.api.renke.domin.UpdateDevice;
import com.fastbee.deviceData.api.renke.domin.UpdateNodeInfo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;

/**
 * 建大仁科气象设备服务
 */
@Service
public class RenkeMetDeviceService {

    /**
     * 获取气象设备全部信息
     * @param deviceStr
     * @param token
     * @return
     */
    public JSONObject getDeviceAllInfo(String deviceStr,String token){

        HttpResponse response = HttpRequest.get("http://api.farm.0531yun.cn/api/v2.0/met/device/getDeviceAllInfo?deviceStr="+deviceStr)
                .header("token", token)
                .execute();
        String respBodyStr = response.body();
        if(StringUtils.isBlank(respBodyStr)) {
            throw new RuntimeException("获取设备实时数据失败!");
        }
        JSONObject jsonObject = JSONUtil.parseObj(respBodyStr);
        return jsonObject;
    }
    /**
     * 根据气象设备地址获取节点信息
     * @param deviceAddr
     * @param token
     * @return
     */
    public  JSONObject getAllInsectEquipmentInformation(String deviceAddr,String token){

        //获取设备实时数据
        //请求参数
        HttpResponse response = HttpRequest.get("http://api.farm.0531yun.cn/api/v2.0/met/device/listTargetNodeInfo?deviceAddr="+deviceAddr)
                .header("token", token)
                .execute();
        String respBodyStr = response.body();
        if(StringUtils.isBlank(respBodyStr)) {
            throw new RuntimeException("获取设备实时数据失败!");
        }
        JSONObject jsonObject = JSONUtil.parseObj(respBodyStr);
        return jsonObject;
    }
    /**
     * 根据气象设备地址获取已启用的节点信息
     * @param deviceAddr
     * @param token
     * @return
     */
    public  JSONObject listTargetEnabledNode(String deviceAddr,String token){

        //获取设备实时数据
        //请求参数
        HttpResponse response = HttpRequest.get("http://api.farm.0531yun.cn/api/v2.0/met/device/listTargetEnabledNode?deviceAddr="+deviceAddr)
                .header("token", token)
                .execute();
        String respBodyStr = response.body();
        if(StringUtils.isBlank(respBodyStr)) {
            throw new RuntimeException("获取设备实时数据失败!");
        }
        JSONObject jsonObject = JSONUtil.parseObj(respBodyStr);
        return jsonObject;
    }
    /**
     * 根据气象节点编号获取遥调信息
     * @param deviceAddr
     * @param token
     * @return
     */
    public  JSONObject listTargetEnabledNode(String deviceAddr,String nodeId,String token){
        // 请求基础URL
        String baseUrl = "http://api.farm.0531yun.cn/api/v2.0/met/device/listTargetEnabledNode";

        StringBuilder queryParams = new StringBuilder();
        if (deviceAddr != null && !deviceAddr.isEmpty()) {
            queryParams.append("deviceAddr=").append(URLEncoder.encode(deviceAddr));
        }
        if (nodeId != null && !nodeId.isEmpty()) {
            if (queryParams.length() > 0) queryParams.append("&");
            queryParams.append("nodeId=").append(URLEncoder.encode(nodeId));
        }
        String fullUrl = baseUrl + "?" + queryParams.toString();
        HttpResponse response = HttpRequest.get(fullUrl)
                .header("token", token)
                .execute();
        String respBodyStr = response.body();
        if(StringUtils.isBlank(respBodyStr)) {
            throw new RuntimeException("获取虫情设备历史记录失败!");
        }
        JSONObject jsonObject = JSONUtil.parseObj(respBodyStr);
        return jsonObject;
    }
    /**
     * 修改指定气象设备全部节点的可用状态
     * @param deviceAddr
     * @param token
     * @return
     */
    public  JSONObject updateAllOfNodesEnable(String deviceAddr,String enable,String token){
        // 请求基础URL
        String baseUrl = "http://api.farm.0531yun.cn/api/v2.0/met/device/updateAllOfNodesEnable";

        StringBuilder queryParams = new StringBuilder();
        if (deviceAddr != null && !deviceAddr.isEmpty()) {
            queryParams.append("deviceAddr=").append(URLEncoder.encode(deviceAddr));
        }
        if (enable != null && !enable.isEmpty()) {
            if (queryParams.length() > 0) queryParams.append("&");
            queryParams.append("enable=").append(URLEncoder.encode(enable));
        }
        String fullUrl = baseUrl + "?" + queryParams.toString();
        HttpResponse response = HttpRequest.get(fullUrl)
                .header("token", token)
                .execute();
        String respBodyStr = response.body();
        if(StringUtils.isBlank(respBodyStr)) {
            throw new RuntimeException("获取虫情设备历史记录失败!");
        }
        JSONObject jsonObject = JSONUtil.parseObj(respBodyStr);
        return jsonObject;
    }

    /**
     * 修改气象设备信息
     * @param updateDevice
     * @param token
     * @return
     */
    public JSONObject updateDevice(UpdateDevice updateDevice, String token){
        //获取设备实时数据
        //请求参数
        Map<String,Object> reqBody =getObjectProperties(updateDevice);
        HttpResponse response = HttpRequest.post("http://api.farm.0531yun.cn/api/v2.0/met/device/updateDevice")
                .header("token", token).body(JSONUtil.toJsonStr(reqBody))
                .execute();
        String respBodyStr = response.body();
        if(StringUtils.isBlank(respBodyStr)) {
            throw new RuntimeException("修改气象设备信息失败!");
        }
        JSONObject jsonObject = JSONUtil.parseObj(respBodyStr);
        return jsonObject;
    }

    /**
     * 更新气象节点信息
     * @param updateNodeInfo
     * @param token
     * @return
     */
    public JSONObject updateNodeInfo(UpdateNodeInfo updateNodeInfo, String token){
        //获取设备实时数据
        //请求参数
        Map<String,Object> reqBody =getObjectProperties(updateNodeInfo);
        HttpResponse response = HttpRequest.post("http://api.farm.0531yun.cn/api/v2.0/met/device/updateNodeInfo")
                .header("token", token).body(JSONUtil.toJsonStr(reqBody))
                .execute();
        String respBodyStr = response.body();
        if(StringUtils.isBlank(respBodyStr)) {
            throw new RuntimeException("修改气象设备信息失败!");
        }
        JSONObject jsonObject = JSONUtil.parseObj(respBodyStr);
        return jsonObject;
    }
    /**
     * 根据条件获取历史数据
     * @param token
     * @return
     */
    public JSONObject getWormDataList(String deviceAddr,String nodeId,String beginTime,String endTime,Integer pages,Integer limit,String token){
        // 请求基础URL
        String baseUrl = "http://api.farm.0531yun.cn/api/v2.0/met/history/getHistoryDataList";

        StringBuilder queryParams = new StringBuilder();
        if (deviceAddr != null && !deviceAddr.isEmpty()) {
            queryParams.append("deviceAddr=").append(URLEncoder.encode(deviceAddr));
        }
        if (beginTime != null && !beginTime.isEmpty()) {
            if (queryParams.length() > 0) queryParams.append("&");
            queryParams.append("beginTime=").append(URLEncoder.encode(beginTime));
        }
        if (endTime != null && !endTime.isEmpty()) {
            if (queryParams.length() > 0) queryParams.append("&");
            queryParams.append("endTime=").append(URLEncoder.encode(endTime));
        }
        if (pages != null) {
            if (queryParams.length() > 0) queryParams.append("&");
            queryParams.append("pages=").append(pages);
        }
        if (limit != null) {
            if (queryParams.length() > 0) queryParams.append("&");
            queryParams.append("limit=").append(limit);
        }
        String fullUrl = baseUrl + "?" + queryParams.toString();
        HttpResponse response = HttpRequest.get(fullUrl)
                .header("token", token)
                .execute();
        String respBodyStr = response.body();
        if(StringUtils.isBlank(respBodyStr)) {
            throw new RuntimeException("获取虫情设备历史记录失败!");
        }
        JSONObject jsonObject = JSONUtil.parseObj(respBodyStr);
        return jsonObject;
    }
    /**
     * 通过反射获取对象的所有属性和值，并存储到Map中
     *
     * @param obj 目标对象
     * @return 包含属性和值的Map
     */
    public static Map<String, Object> getObjectProperties(Object obj) {
        Map<String, Object> propertiesMap = new HashMap<>();

        // 获取对象的类类型
        Class<?> clazz = obj.getClass();

        // 获取类的所有字段（包括私有字段）
        Field[] fields = clazz.getDeclaredFields();

        // 遍历字段
        for (Field field : fields) {
            // 设置字段为可访问（即使它是私有的）
            field.setAccessible(true);

            try {
                // 获取字段的名称
                String fieldName = field.getName();

                // 获取字段的值（从目标对象中）
                Object fieldValue = field.get(obj);

                // 将字段名称和值存入Map中
                propertiesMap.put(fieldName, fieldValue);

            } catch (IllegalAccessException e) {
                // 理论上不应该发生，因为我们已经设置了setAccessible(true)
                e.printStackTrace();
            }
        }
        return propertiesMap;
    }
}
